#include <gtk/gtk.h>
#include "SDL.h"

#define STEP 2

int MAX_X,MAX_Y; /* These are set to the width of the screen - the size
			of the cursor once we know how big the screen is */
int MIN_X = 0;
int MIN_Y = 0;
SDL_Surface *screen, *cursor, *buff;
SDL_Rect back, cur;

/* Our function to exit both programs nicely */
int destroy() {
   gtk_main_quit();
   SDL_Quit();
   return 0;
}

/* Our functions for movement called by both gtk and SDL */
int do_up() {
   SDL_Rect old[2];

   old[0] = back;
   SDL_BlitSurface(buff, &cur,screen,&back);
   back.y -= STEP;
   if (back.y < MIN_Y) {
      back.y = MIN_Y;
   }
   SDL_BlitSurface(screen,&back, buff, &cur);
   SDL_BlitSurface(cursor,&cur,screen,&back);
   old[1] = back;
   SDL_UpdateRects(screen,2,old);
   return 1;
}

int do_down() {
   SDL_Rect old[2];

   old[0] = back;
   SDL_BlitSurface(buff, &cur,screen,&back);
   back.y += STEP;
   if (back.y > MAX_Y) {
      back.y = MAX_Y;
   }
   SDL_BlitSurface(screen,&back, buff, &cur);
   SDL_BlitSurface(cursor,&cur,screen,&back);
   old[1] = back;
   SDL_UpdateRects(screen,2,old);
   return 1;
}

int do_left() {
   SDL_Rect old[2];

   old[0] = back;
   SDL_BlitSurface(buff, &cur,screen,&back);
   back.x -= STEP;
   if (back.x < MIN_X) {
      back.x = MIN_X;
   }
   SDL_BlitSurface(screen,&back, buff, &cur);
   SDL_BlitSurface(cursor,&cur,screen,&back);
   old[1] = back;
   SDL_UpdateRects(screen,2,old);
   return 1;
}

int do_right() {
   SDL_Rect old[2];

   old[0] = back;
   SDL_BlitSurface(buff, &cur,screen,&back);
   back.x += STEP;
   if (back.x > MAX_X) {
      back.x = MAX_X;
   }
   SDL_BlitSurface(screen,&back, buff, &cur);
   SDL_BlitSurface(cursor,&cur,screen,&back);
   old[1] = back;
   SDL_UpdateRects(screen,2,old);
   return 1;
}

/* Setup the gtk interface */
int mygtk_init(int argc, char *argv[]) {
   GtkWidget *window, *qbutton, *lbutton, *rbutton, *ubutton, *dbutton,
      *movements, *bigbox, *sidebox;
   
   /* Setup the basic window */
   gtk_init(&argc, &argv);
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

   gtk_signal_connect (GTK_OBJECT (window), "destroy", 
   GTK_SIGNAL_FUNC (destroy), NULL);

   /* Create some buttons and boxes for packing nicely */
   movements = gtk_vbox_new(FALSE, 0);
   sidebox = gtk_hbox_new(FALSE, 0);
   bigbox = gtk_hbox_new(FALSE, 0);
   qbutton = gtk_button_new_with_label("Quit");
   lbutton = gtk_button_new_with_label("Left");
   rbutton = gtk_button_new_with_label("Right");
   ubutton = gtk_button_new_with_label("UP");
   dbutton = gtk_button_new_with_label("Down");

   /* Add callbacks to our buttons we have created so they actually
      do something */
   gtk_signal_connect_object (GTK_OBJECT (qbutton), "clicked",
      GTK_SIGNAL_FUNC (destroy), NULL);
   gtk_signal_connect_object (GTK_OBJECT (lbutton), "clicked",
      GTK_SIGNAL_FUNC (do_left), NULL);
   gtk_signal_connect_object (GTK_OBJECT (rbutton), "clicked",
      GTK_SIGNAL_FUNC (do_right), NULL);
   gtk_signal_connect_object (GTK_OBJECT (ubutton), "clicked",
      GTK_SIGNAL_FUNC (do_up), NULL);
   gtk_signal_connect_object (GTK_OBJECT (dbutton), "clicked",
      GTK_SIGNAL_FUNC (do_down), NULL);

   /* Pack our widgets */
   gtk_box_pack_start (GTK_BOX (sidebox), lbutton, TRUE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (sidebox), rbutton, TRUE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (movements), ubutton, TRUE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (movements), sidebox, TRUE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (movements), dbutton, TRUE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (bigbox), movements, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (bigbox), qbutton, TRUE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), bigbox);

   /* Display the widgets */
   gtk_widget_show (ubutton);
   gtk_widget_show (dbutton);
   gtk_widget_show (lbutton);
   gtk_widget_show (rbutton);
   gtk_widget_show (sidebox);
   gtk_widget_show (movements);
   gtk_widget_show (qbutton);
   gtk_widget_show (bigbox);
   gtk_widget_show (window);
   return 1;
}

/* Setup the SDL surface and load the cursor */
int mysdl_init() {
   Uint16 width = 640;
   Uint16 height = 480;
   Uint8 bits = 8;

   if ( SDL_Init(SDL_INIT_EVERYTHING) < 0 ) {
      fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
      return 0;
   }
   atexit(destroy);
   SDL_WM_SetCaption("Demo using gtk", "gtkdemo");
   screen = SDL_SetVideoMode(width, height, bits, SDL_SWSURFACE|SDL_ANYFORMAT);
   if ( screen == NULL ) {
      fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
      return 0;
   }
   myclear_screen(1,1,1);   

   /*setup the cursor */
   cursor = SDL_LoadBMP("cursor.bmp");
   buff = SDL_AllocSurface(SDL_SWSURFACE,cursor->w,cursor->h,8,0,0,0,0);
   if (cursor == NULL) {
      fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
      return 0;
   }
   /* inialize our palette */
   if ( cursor->format->palette ) {
      SDL_SetColors(screen, cursor->format->palette->colors, 0,
         cursor->format->palette->ncolors);
   }

   /* setup our buffers so that we can blit nicely, the gist of this is
      copy the background to buff so we can copy it back when we move
      the cursor.  Then copy the cursor to the background */
   MAX_X = screen->w - cursor->w;
   MAX_Y = screen->h - cursor->h;
   MIN_X = 0;
   MIN_Y = 0;
   back.h = cursor->h;
   back.w = cursor->w;
   cur.h = back.h;
   cur.w = back.w;

   back.x = screen->h/2;
   back.y = screen->w/2;

   cur.x = 0;
   cur.y = 0;

   /* Do our initall backup and copy stuff */
   SDL_BlitSurface(screen, &back, buff, &cur);
   SDL_BlitSurface(cursor, &cur, screen, &back);
   SDL_Flip(screen);
   return 1;
}

int myclear_screen(int r, int g, int b) {
   Uint32 mypixel;

   mypixel = SDL_MapRGB(screen->format, r, g, b);
   SDL_FillRect(screen,NULL,mypixel);
   SDL_Flip(screen);
   return 1;
}

/* Event loop for SDL which is a function that gets called from gtk */
int mySDLmain_loop() {
   int done = 0;
   Uint8 *keys;
   SDL_Event event;

   if (SDL_PollEvent(&event) == 0) {
   } else {
      if (event.type == SDL_KEYDOWN) {
         keys = SDL_GetKeyState(NULL);
         if (keys[SDLK_ESCAPE] == SDL_PRESSED) {
            return destroy();
         }
         if (keys[SDLK_UP] == SDL_PRESSED) {
            do_up();
         }
         if (keys[SDLK_DOWN] == SDL_PRESSED) {
            do_down();
         }
         if (keys[SDLK_LEFT] == SDL_PRESSED) {
            do_left();
         }
         if (keys[SDLK_RIGHT] == SDL_PRESSED) {
           do_right();
         }
      } else if (event.type == SDL_QUIT) {
         return destroy();
      }
   }
   return 1;
}


int main(int argc, char *argv[]) {

   /* First initialize gtk, so we know we're running under X11, and
      SDL isn't going to use a fullscreen driver. */
   if (!mygtk_init(argc, argv)) {
      return 0;
   }
   if (!mysdl_init()) {
      return 0;
   }
   /* Add our SDL loop as a function the gtk needs to call every once and
      awhile.  Note there are two ways todo this try them both out */
   gtk_idle_add(mySDLmain_loop,NULL); 
   /* gtk_timeout_add(30,mySDLmain_loop,NULL); */

   /* Startup gtk's event loop */
   gtk_main();
   exit(1);
}
